<?php
namespace OrganizationsManagement\Controller;

use OrganizationsManagement\Util\Utilities;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource;
use OrganizationsManagement\Service;

class DataExportation extends Simple
{
    /** @var Service\DataExportation $serviceDataExportation */
    protected $serviceDataExportation;

    public function __construct(Service\DataExportation $dataExportation) {
        $this->serviceDataExportation = $dataExportation;
    }

    /**
     * @param DTO\Request\Filter $request
     * @param DTO\Response $response
     */
    public function getDataExportationInfo(DTO\Request\Filter $request, DTO\Response $response){
        try {
            $filters  = $request->getFilterCriteria()->getConditions();
            $layoutId = Utilities::getValueInFilterDataSourceByName($filters, 'LAYOUT_CODE');
            $originOrganization  = Utilities::getValueInFilterDataSourceByName($filters, 'ORIGIN_ORG');
            $destinyOrganization = Utilities::getValueInFilterDataSourceByName($filters, 'DESTINY_ORG');
            $destinyIdOrgSaas    = Utilities::getValueInFilterDataSourceByName($filters, 'DESTINY_IDORGSAAS');
            $dataExportation = $this->serviceDataExportation->getDataExportationInfo($layoutId, $originOrganization,
                $destinyOrganization, $destinyIdOrgSaas);
            $response->addDataSet(new DataSource\DataSet('dataExportationInfo', $dataExportation));
        } catch(\Exception $ex) {
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    /**
     * @param DTO\Request\DataSet $request
     * @param DTO\Response $response
     */
    public function exportTable(DTO\Request\Dataset $request, DTO\Response $response){
        try {
            $row = $request->getDataSet()->getRows()[0];
            $tableName         = $row['TABLE_NAME'];
            $originOrg         = $row['ORIGIN_ORG'];
            $destinyOrg        = $row['DESTINY_ORG'];
            $destinyIdOrgSaas  = $row['DESTINY_IDORGSAAS'];
            $exportTable = $this->serviceDataExportation->exportTable($tableName, $originOrg, $destinyOrg, $destinyIdOrgSaas);
            $response->addDataSet(new DataSource\DataSet('exportedTable', $exportTable));
        } catch(\Exception $ex) {
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    /**
     * @param DTO\Request\DataSet $request
     * @param DTO\Response $response
     */
    public function revertTableExportation(DTO\Request\Dataset $request, DTO\Response $response){
        try {
            $row = $request->getDataSet()->getRows()[0];
            $tableName   = $row['TABLE_NAME'];
            $destinyOrg  = $row['DESTINY_ORG'];
            $exportTable = $this->serviceDataExportation->revertTableExportation($tableName, $destinyOrg);
            $response->addDataSet(new DataSource\DataSet('revertedTable', $exportTable));
        } catch(\Exception $ex) {
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    /**
     * @param DTO\Request\DataSet $request
     * @param DTO\Response $response
     */
    public function createDataExportationLog(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $row = $request->getDataSet()->getRows()[0];
            $fileName    = $row['FILE_NAME'];
            $logContent  = $row['LOG_CONTENT'];
            $logResponse = $this->serviceDataExportation->createDataExportationLog($fileName, $logContent);
            $response->addDataSet(new DataSource\DataSet('logCreated', $logResponse));
        } catch(\Exception $ex){
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    /**
     * @param DTO\Request\DataSet $request
     * @param DTO\Response $response
     */
    public function downloadDataExportationLog(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $row = $request->getDataSet()->getRows()[0];
            $fileName = $row['FILE_NAME'];
            $this->serviceDataExportation->downloadDataExportationLog($fileName);
        } catch(\Exception $ex){
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }
}